__title__ = 'kaptan'
__package_name__ = 'kaptan'
__version__ = '0.6.0'
__description__ = 'Configuration manager'
__email__ = 'mail@emreyilmaz.me'
__url__ = 'https://github.com/emre/kaptan'
__author__ = 'Emre Yilmaz'
__license__ = 'BSD'
__copyright__ = 'Copyright 2013-2019 Emre Yilmaz'
